import { createStore } from 'vuex'
import { 
	getPhotoLogList,
	getPhotoLogTemplateList,
	deletePhotoLog,
	deletePhotoLogTemplate,
	addNewPhotoLog,
	addNewPhotoLogTemplate,
	getPhotoLog,
	updatePhotoLog,
	updateLogTemplate,
	getPhotoGroups,
	getPhotoTags,
	getPhotosByGroup,
	updatePhoto,
	deletePhoto,
	cropPhoto,
} from '@/api'

export default createStore({
	state: {
		photoLogList: [],
		photoLogTemplateList: [],
		photoLogs: [],
		photoGroups: null,
		photoTags: null,
		photos: []
	},
	mutations: {
		SET_PHOTO_LOG_LIST(state, newPhotoLogList) {
			state.photoLogList = newPhotoLogList
		},
		SET_PHOTO_LOG_TEMPLATE_LIST(state, newPhotoLogTemplateList) {
			state.photoLogTemplateList = newPhotoLogTemplateList
		},
		SET_PHOTO_LOG(state, photoLog) {
			let log_index = state.photoLogs.findIndex(log => log.id === photoLog.id)
			if (log_index > -1) {
				state.photoLogs[log_index] = photoLog
			} else {
				state.photoLogs.push(photoLog)
			}
		},
		SET_PHOTO_LOG_TEMPLATE(state, photoLogtemplate) {
			let log_index = state.photoLogTemplateList.findIndex(log => log.id === photoLogtemplate.id)
			if (log_index > -1) {
				state.photoLogTemplateList[log_index] = photoLogtemplate
			} else {
				state.photoLogTemplateList.push(photoLogtemplate)
			}
		},
		REMOVE_PHOTO_LOG(state, id) {
			let log_index = state.photoLogList.findIndex(log => log.id === id)
			state.photoLogList.splice(log_index, 1)
		},
		REMOVE_PHOTO_LOG_TEMPLATE(state, id) {
			let log_index = state.photoLogTemplateList.findIndex(log => log.id === id)
			state.photoLogTemplateList.splice(log_index, 1)
		},
		SET_PHOTO_GROUPS(state, groups) {
			state.photoGroups = groups
		},
		SET_PHOTO_TAGS(state, tags) {
			state.photoTags = tags
		},
		SET_PHOTOS_IN_GROUP(state, {group_id, photos}) {
			if (group_id in state.photos) {
				state.photos.group_id = photos
			} else {
				state.photos = {...state.photos, [group_id]: photos}
			}
		},
		UPDATE_PHOTO(state, photo) {
			for (const idx in state.photos) {
				let group = state.photos[idx]
				let photo_index = group.findIndex(p => p.id == photo.id)
				if (photo_index > -1) {
					let state_photo = state.photos[idx].splice(photo_index, 1)[0]
					Object.keys(photo).forEach(key => {
						state_photo[key] = photo[key]
					})
					if (photo['group']) {
						if (!state.photos[photo.group] || state.photos[photo.group].length === 0) {
							state.photos[photo.group] = []
						}
						state.photos[state_photo.group].push(state_photo)
					} else {
						state.photos[state_photo.group].splice(photo_index, 0, state_photo)
					}
					break
				}
			}
		},
		DELETE_PHOTO(state, id) {
			for (const group_id in state.photos) {
				let group = state.photos[group_id]
				let photo_index = group.findIndex(p => p.id == id)
				if (photo_index > -1) {
					state.photos[group_id].splice(photo_index, 1)
				}
			}
		}
	},
	actions: {
		loadPhotoLogList({commit}) {
			return new Promise((resolve, reject) => {
				getPhotoLogList().then((response) => {
					commit('SET_PHOTO_LOG_LIST', response.data)
					resolve()
				}).catch((error) => {
					console.log(error)
					reject()
				})
			})
		},
		loadPhotoLogTemplateList({commit}) {
			return new Promise((resolve, reject) => {
				getPhotoLogTemplateList().then((response) => {
					commit('SET_PHOTO_LOG_TEMPLATE_LIST', response.data)
					resolve()
				}).catch((error) => {
					console.log(error)
					reject()
				})
			})
		},
		loadPhotoLog({commit}, id) {
			return new Promise((resolve, reject) => {
				getPhotoLog(id).then((response) => {
					commit('SET_PHOTO_LOG', response.data)
					resolve()
				}).catch((error) => {
					console.log(error)
					reject()
				})
			})
		},
		deletePhotoLog({commit}, id) {
			deletePhotoLog(id).then(() => {
				commit('REMOVE_PHOTO_LOG', id)
			}).catch((error) => {
				console.log(error)
			})
		},
		deletePhotoLogTemplate({commit}, id) {
			deletePhotoLogTemplate(id).then(() => {
				commit('REMOVE_PHOTO_LOG_TEMPLATE', id)
			}).catch((error) => {
				console.log(error)
			})
		},
		addNewPhotoLog({dispatch}, {title, date}) {
			return new Promise((resolve, reject) => {
				addNewPhotoLog(title, date).then((response) => {
					dispatch('loadPhotoLogList')
					resolve(response.data.id)
				}).catch((error) => {
					console.log(error)
					reject(error)
				})
			})
		},
		addNewPhotoLogTemplate({dispatch}, {title, date}) {
			return new Promise((resolve, reject) => {
				addNewPhotoLogTemplate(title, date).then((response) => {
					dispatch('loadPhotoLogTemplateList')
					resolve(response.data.id)
				}).catch((error) => {
					console.log(error)
					reject(error)
				})
			})
		},
		updatePhotoLogDetails({commit}, photolog) {
			if (photolog !== null) {
				commit('SET_PHOTO_LOG', photolog)
				return updatePhotoLog(photolog)
			}
		},
		updatePhotoLogTemplate({commit}, photologtemplate) {
			if (photologtemplate !== null) {
				commit('SET_PHOTO_LOG_TEMPLATE', photologtemplate)
				return updateLogTemplate(photologtemplate)
			}
		},
		loadPhotoGroups({commit}) {
			return new Promise((resolve, reject) => {
				getPhotoGroups().then((response) => {
					commit('SET_PHOTO_GROUPS', response.data)
					resolve(response.data)
				}).catch((error) => {
					reject(error)
				})
			})
		},
		loadPhotoTags({commit}) {
			return new Promise((resolve, reject) => {
				getPhotoTags().then((response) => {
					commit('SET_PHOTO_TAGS', response.data)
					resolve(response.data)
				}).catch((error) => {
					reject(error)
				})
			})
		},
		loadPhotosInGroup({commit}, group_id) {
			return new Promise((resolve, reject) => {
				getPhotosByGroup(group_id).then((response) => {
					commit('SET_PHOTOS_IN_GROUP', {group_id, photos: response.data})
					resolve(response.data)
				}).catch((error) => {
					reject(error)
				})
			})
		},
		loadPhotosInAllGroups({dispatch, getters}) {
			for (const index in getters.photoGroups) {
				const group_id = getters.photoGroups[index].id
				dispatch('loadPhotosInGroup', group_id)
			}
		},
		updatePhoto({commit}, photo) {
			new Promise((resolve, reject) => {
				updatePhoto(photo).then(() => {
					commit('UPDATE_PHOTO', photo)
					resolve()
				}).catch((error) => {
					reject(error)
				})
			})
		},
		deletePhoto({commit}, id) {
			new Promise((resolve, reject) => {
				deletePhoto(id).then(() => {
					commit('DELETE_PHOTO', id)
					resolve()
				}).catch((error) => {
					reject(error)
				})
			})
		},
		cropPhoto({commit}, {id, mode}) {
			new Promise((resolve, reject) => {
				cropPhoto(id, mode).then((response) => {
					commit('UPDATE_PHOTO', {
						id: response.data.id,
						cropped_image: response.data.cropped_image
					})
					resolve()
				}).catch((error) => {
					reject(error)
				})
			})
		},
	},
	getters: {
		photoLogById: (state) => (id) =>  {
			let log = state.photoLogList.filter(log => log.id == id)
			if (log.length > 0) {
				return log[0]
			}
			return null
		},
		photoLogTemplateById: (state) => (id) =>  {
			let log = state.photoLogTemplateList.filter(log => log.id == id)
			if (log.length > 0) {
				return log[0]
			}
			return null
		},
		photoLogDetailsById: (state) => (id) =>  {
			let log = state.photoLogs.filter(log => log.id == id)
			if (log.length > 0) {
				return log[0]
			}
			return null
		},
		photoGroups (state) {
			return state.photoGroups
		},
		photos (state) {
			return state.photos
		},
		photoTags(state) {
			return state.photoTags
		},
		photoById: (state) => (id) =>  {
			for (const group_idx in state.photos) {
				let found_photo = state.photos[group_idx].filter(p => p.id == id)
				if (found_photo.length > 0) {
					return found_photo[0]
				}
			}
			return null
		},
	},
	modules: {},
})
